#include <sys/param.h>
#include <sys/time.h>

#include <fcntl.h>
#include <kvm.h>
#include <nlist.h>
#include <stdio.h>
#include "testvn.h"

static void fetchinfo(struct testvn *);
static void printinfo(struct testvn *);
static void printheader(void);

static	char buf[26];

kvm_t *kd;

#define	X_DESIREDVNODES	0
#define	X_NUMVNODES	1
#define	X_FREEVNODES	2

static struct nlist namelist[] = {
	{ "_desiredvnodes" },
	{ "_numvnodes" },
	{ "_freevnodes" },
	{ "" },
};

int
main(argc, argv)
int	argc;
char	*argv[];
{
	struct testvn *set;
	char errbuf[80];
	int error;

	kd = kvm_openfiles(NULL, NULL, NULL, O_RDONLY, errbuf);
	if (kvm_nlist(kd, namelist)) {
		fprintf(stderr, "%s: kvm_openfiles: %s\n", argv[0], errbuf);
		exit(1);
	}

	error = usr_setup(argc, argv);
	if (error)
		goto err;

	printheader();
	for (set = usr_tests; set->test_name != NULL; set++) {
		(*(set->test_func))();
		fetchinfo(set);
	}
	for (set = usr_tests; set->test_name != NULL; set++)
		printinfo(set);

	usr_teardown();
err:
	kvm_close(kd);
	exit (error);
}

static void
printheader(void)
{
	time_t now;
	struct tm *tp;

	time(&now);
	tp = localtime(&now);
	(void) strftime(buf, sizeof(buf), "%c", tp);
	buf[16] = '\0';
	fprintf(stdout, "%s\n", buf);

	fprintf(stdout, "%8s", "Desired");
	fprintf(stdout, "|%8s", "Num");
	fprintf(stdout, "|%8s", "Free");
	fprintf(stdout, "|\t%s\n", "Test name");
}

static void
printinfo(s)
struct testvn	*s;
{
	fprintf(stdout, "%8d", s->desiredvnodes);
	fprintf(stdout, "|%8d", s->numvnodes);
	fprintf(stdout, "|%8d", s->freevnodes);
	fprintf(stdout, "|\t%s\n", s->test_name);
}


static void
fetchinfo(s)
struct testvn	*s;
{
	NREAD(X_DESIREDVNODES, &s->desiredvnodes, LONG);
	NREAD(X_NUMVNODES, &s->numvnodes, LONG);
	NREAD(X_FREEVNODES, &s->freevnodes, LONG);
}


int
kvm_ckread(a, b, l)
	void *a, *b;
	int l;
{
	if (kvm_read(kd, (u_long)a, b, l) != l) {
		printf("Can't read kvm\n");
		return (0);
	}
	else
		return (1);
}
